@@ -67,7 +67,7 @@ std::pair<bool, long> RingZZ::coerceToLongInteger(ring_elem a) const
                                mpz_get_si(a.get_mpz()));
 }
 
-ring_elem RingZZ::random() const { return ring_elem(rawRandomInteger(0)); }
+ring_elem RingZZ::random() const { return ring_elem(rawRandomInteger(nullptr)); }
 
 void RingZZ::elem_text_out(buffer &o,
                            const ring_elem ap,
