@@ -3,17 +3,16 @@
 #ifndef _dmat_gf_flint_big_hpp_
 #define _dmat_gf_flint_big_hpp_
 
+#include <utility>                 // for swap
+#include "aring-gf-flint-big.hpp"  // for ARingGFFlintBig
+
 // The following needs to be included before any flint files are included.
 #include <M2/gc-include.h>
 
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wconversion"
-#include <flint/arith.h>
-#include <flint/nmod_mat.h>
-#include <flint/fmpq_mat.h>
-#include <flint/fq_nmod_mat.h>
+#include <flint/fq_nmod_mat.h>  // for fq_nmod_mat_t, fq_nmod_mat_entry, ...
 #pragma GCC diagnostic pop
-#include "aring-gf-flint-big.hpp"
 
 template <typename ACoeffRing>
 class DMat;
