@@ -327,17 +327,12 @@ bool DMatLinAlg<RingType>::solve(const Mat& B, Mat& X)
   const std::vector<size_t>& perm = mLUObject.permutation();
   size_t rk = pivotColumns.size();
 
-  ElementType tmp, tmp2;
-  ring().init(tmp);
-  ring().init(tmp2);
+  typename RingType::Element tmp(ring()), tmp2(ring());
 
-  ElementType* b = newarray(ElementType, LU.numRows());
-  ElementType* y = newarray(ElementType, rk);
-  ElementType* x = newarray(ElementType, LU.numColumns());
-
-  for (size_t i = 0; i < LU.numRows(); i++) ring().init(b[i]);
-  for (size_t i = 0; i < rk; i++) ring().init(y[i]);
-  for (size_t i = 0; i < LU.numColumns(); i++) ring().init(x[i]);
+  typedef typename RingType::ElementArray ElementArray;
+  ElementArray b(ring(), LU.numRows());
+  ElementArray y(ring(), rk);
+  ElementArray x(ring(), LU.numColumns());
 
   X.resize(LU.numColumns(), B.numColumns());
 
@@ -379,12 +374,6 @@ bool DMatLinAlg<RingType>::solve(const Mat& B, Mat& X)
             }
           if (!ring().is_zero(tmp))
             {
-              // Cleanup, and return false
-              freemem(b);
-              freemem(y);
-              freemem(x);
-              ring().clear(tmp);
-              ring().clear(tmp2);
               // printf("returning false\n");
               return false;
             }
@@ -418,15 +407,7 @@ bool DMatLinAlg<RingType>::solve(const Mat& B, Mat& X)
       /// displayMat(o, X);
       /// printf("%s\n", o.str());
     }
-  ring().clear(tmp);
-  ring().clear(tmp2);
 
-  for (size_t i = 0; i < LU.numRows(); i++) ring().clear(b[i]);
-  for (size_t i = 0; i < rk; i++) ring().clear(y[i]);
-  for (size_t i = 0; i < LU.numColumns(); i++) ring().clear(x[i]);
-  freemem(b);
-  freemem(y);
-  freemem(x);
   return true;  // The system seems to have been consistent
 }
 
@@ -565,10 +546,7 @@ size_t DMatLinAlg<RingType>::kernel(Mat& X)
   const Mat& LU = mLUObject.LUinPlace();
   const std::vector<size_t>& pivotColumns = mLUObject.pivotColumns();
 
-  // THIS SHOULD NOT BE SO PAINFUL!!
-  ElementType tmp, tmp2;
-  ring().init(tmp);
-  ring().init(tmp2);
+  typename RingType::Element tmp(ring()), tmp2(ring());
 
   // First, let's set X to be the correct size:
   X.resize(LU.numColumns(), LU.numColumns() - pivotColumns.size());
@@ -610,9 +588,6 @@ size_t DMatLinAlg<RingType>::kernel(Mat& X)
       colX++;
       col++;
     }
-
-  ring().clear(tmp);
-  ring().clear(tmp2);
   return X.numColumns();
 }
 
