@@ -3,18 +3,17 @@
 #ifndef _dmat_zzp_flint_hpp_
 #define _dmat_zzp_flint_hpp_
 
+#include <utility>              // for swap
+#include "aring-zzp-flint.hpp"  // for ARingZZpFlint
+
 // The following needs to be included before any flint files are included.
 #include <M2/gc-include.h>
 
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wconversion"
-#include <flint/arith.h>
-#include <flint/nmod_mat.h>
-#include <flint/fmpq_mat.h>
+#include <flint/nmod_mat.h>  // for nmod_mat_t, nmod_mat_clear, nmod_mat_init, nmod_m...
 #pragma GCC diagnostic pop
 
-#include "aring-zzp-flint.hpp"
-
 template <typename ACoeffRing>
 class DMat;
 
