@@ -62,8 +62,8 @@ class MonomialInfo : public our_new_delete
   mutable unsigned long ncalls_compare;
   mutable unsigned long ncalls_mult;
   mutable unsigned long ncalls_get_component;
-  mutable unsigned long ncalls_from_exponent_vector;
-  mutable unsigned long ncalls_to_exponent_vector;
+  mutable unsigned long ncalls_from_expvector;
+  mutable unsigned long ncalls_to_expvector;
   mutable unsigned long ncalls_to_varpower;
   mutable unsigned long ncalls_from_varpower;
   mutable unsigned long ncalls_is_equal;
@@ -107,13 +107,13 @@ class MonomialInfo : public our_new_delete
     return m[1];
   }
 
-  bool from_exponent_vector(const_ntuple_monomial e,
+  bool from_expvector(const_ntuple_monomial e,
                             long comp,
                             packed_monomial result) const
   {
     // Pack the vector e[0]..e[nvars-1],comp.  Create the hash value at the same
     // time.
-    ncalls_from_exponent_vector++;
+    ncalls_from_expvector++;
     result[0] = 0;
     result[1] = comp;
 
@@ -162,12 +162,12 @@ class MonomialInfo : public our_new_delete
     return true;
   }
 
-  bool to_exponent_vector(const_packed_monomial m,
+  bool to_expvector(const_packed_monomial m,
                           ntuple_monomial result,
                           long &result_comp) const
   {
     // Unpack the monomial m.
-    ncalls_to_exponent_vector++;
+    ncalls_to_expvector++;
     result_comp = m[1];
     m += 2 + nweights;
     for (int i = 0; i < nvars; i++) *result++ = *m++;
@@ -181,7 +181,7 @@ class MonomialInfo : public our_new_delete
     // Unpack the monomial m into result, which should already be allocated
     // 0..nvars-1
     // this is to connect with older 'int *' monomials.
-    ncalls_to_exponent_vector++;
+    ncalls_to_expvector++;
     result_comp = static_cast<int>(m[1]);
     m += 2;
     for (int i = 0; i < nvars; i++) *result++ = static_cast<int>(*m++);
