@@ -18,8 +18,8 @@ binomial_ring::binomial_ring(const PolynomialRing *RR, int *wts, bool revlex0)
     : R(RR),
       F(RR->make_FreeModule(1)),
       nvars(RR->n_vars()),
-      have_weights(wts != NULL),
-      weights(NULL),
+      have_weights(wts != nullptr),
+      weights(nullptr),
       revlex(revlex0)
 {
   int i;
@@ -53,9 +53,9 @@ binomial_ring::~binomial_ring()
 
 void binomial_ring::remove_monomial(monomial &m) const
 {
-  if (m == NULL) return;
+  if (m == nullptr) return;
   monstash->delete_elem(m);
-  m = NULL;
+  m = nullptr;
 }
 
 monomial binomial_ring::new_monomial() const
@@ -284,7 +284,7 @@ void binomial_ring::translate_monomial(const binomial_ring *old_ring,
                                        monomial &m) const
 {
   int i;
-  if (m == NULL) return;
+  if (m == nullptr) return;
   monomial result = new_monomial();
   for (i = 0; i < old_ring->nvars; i++) result[i] = m[i];
   for (i = old_ring->nvars; i < nvars; i++) result[i] = 0;
@@ -302,7 +302,7 @@ void binomial_ring::translate_binomial(const binomial_ring *old_ring,
 
 vec binomial_ring::monomial_to_vector(monomial m) const
 {
-  if (m == NULL) return NULL;
+  if (m == nullptr) return nullptr;
   ring_elem f = R->make_logical_term(
       R->getCoefficients(), R->getCoefficients()->one(), m);
   return R->make_vec(0, f);
@@ -339,9 +339,9 @@ bool binomial_ring::vector_to_binomial(vec f, binomial &result) const
 // result should already have both monomials allocated
 // returns false if f is not a binomial, otherwise result is set.
 {
-  if (f == NULL) return false;
+  if (f == nullptr) return false;
   Nterm *t = f->coeff;
-  if (t == NULL || t->next == NULL || t->next->next != NULL) return false;
+  if (t == nullptr || t->next == nullptr || t->next->next != nullptr) return false;
 
   R->getMonoid()->to_expvector(t->monom, result.lead);
   set_weights(result.lead);
@@ -362,7 +362,7 @@ void binomial_ring::intvector_to_binomial(vec f, binomial &result) const
       result.tail[i] = 0;
     }
 
-  for (; f != NULL; f = f->next)
+  for (; f != nullptr; f = f->next)
     {
       std::pair<bool, long> res = globalZZ->coerceToLongInteger(f->coeff);
       assert(res.first);
@@ -418,7 +418,7 @@ bool binomial_ring::calc_s_pair(binomial_s_pair &s, binomial &result) const
 
 void binomial_ring::monomial_out(buffer &o, const_exponents m) const
 {
-  if (m == NULL) return;
+  if (m == nullptr) return;
   gc_vector<int> vp;
   varpower::from_expvector(nvars, m, vp);
   monomial n = R->getMonoid()->make_one();
@@ -430,7 +430,7 @@ void binomial_ring::monomial_out(buffer &o, const_exponents m) const
 void binomial_ring::elem_text_out(buffer &o, const binomial &f) const
 {
   monomial_out(o, f.lead);
-  if (f.tail == NULL) return;
+  if (f.tail == nullptr) return;
   o << "-";
   monomial_out(o, f.tail);
 }
@@ -439,7 +439,7 @@ void binomial_ring::elem_text_out(buffer &o, const binomial &f) const
 ///////////////////////
 
 binomial_s_pair_set::binomial_s_pair_set(const binomial_ring *RR)
-    : R(RR), _prev_lcm(NULL), _n_elems(0), _max_degree(0)
+    : R(RR), _prev_lcm(nullptr), _n_elems(0), _max_degree(0)
 {
   _pairs = new s_pair_degree_list;  // list header
   _npairs.push_back(0);
@@ -452,17 +452,17 @@ void binomial_s_pair_set::enlarge(const binomial_ring *newR)
   R = newR;
 
   old_ring->remove_monomial(_prev_lcm);
-  _prev_lcm = NULL;
-  for (s_pair_degree_list *thisdeg = _pairs->next; thisdeg != NULL;
+  _prev_lcm = nullptr;
+  for (s_pair_degree_list *thisdeg = _pairs->next; thisdeg != nullptr;
        thisdeg = thisdeg->next)
-    for (s_pair_lcm_list *thislcm = thisdeg->pairs; thislcm != NULL;
+    for (s_pair_lcm_list *thislcm = thisdeg->pairs; thislcm != nullptr;
          thislcm = thislcm->next)
       R->translate_monomial(old_ring, thislcm->lcm);
 }
 
 void binomial_s_pair_set::remove_lcm_list(s_pair_lcm_list *p)
 {
-  while (p->pairs != NULL)
+  while (p->pairs != nullptr)
     {
       s_pair_elem *thispair = p->pairs;
       p->pairs = thispair->next;
@@ -473,7 +473,7 @@ void binomial_s_pair_set::remove_lcm_list(s_pair_lcm_list *p)
 }
 void binomial_s_pair_set::remove_pair_list(s_pair_degree_list *p)
 {
-  while (p->pairs != NULL)
+  while (p->pairs != nullptr)
     {
       s_pair_lcm_list *thislcm = p->pairs;
       p->pairs = thislcm->next;
@@ -483,7 +483,7 @@ void binomial_s_pair_set::remove_pair_list(s_pair_degree_list *p)
 }
 binomial_s_pair_set::~binomial_s_pair_set()
 {
-  while (_pairs != NULL)
+  while (_pairs != nullptr)
     {
       s_pair_degree_list *thisdeg = _pairs;
       _pairs = thisdeg->next;
@@ -500,13 +500,13 @@ void binomial_s_pair_set::insert_pair(s_pair_degree_list *q, binomial_s_pair &s)
   s_pair_lcm_list *r = &head;
   while (true)
     {
-      if (r->next == NULL || ((cmp = R->compare(s.lcm, r->next->lcm)) == GT))
+      if (r->next == nullptr || ((cmp = R->compare(s.lcm, r->next->lcm)) == GT))
         {
           // Insert new lcm node
           s_pair_lcm_list *r1 = new s_pair_lcm_list;
           r1->next = r->next;
           r1->lcm = s.lcm;
-          r1->pairs = NULL;
+          r1->pairs = nullptr;
           r->next = r1;
           break;
         }
@@ -527,7 +527,7 @@ void binomial_s_pair_set::insert_pair(s_pair_degree_list *q, binomial_s_pair &s)
 void binomial_s_pair_set::insert(binomial_gb_elem *p)
 {
   monomial lcm = R->make_monomial(R->lead_monomial(p->f));
-  binomial_s_pair s(p, NULL, lcm);
+  binomial_s_pair s(p, nullptr, lcm);
   insert(s);
 }
 
@@ -548,13 +548,13 @@ void binomial_s_pair_set::insert(binomial_s_pair s)
   s_pair_degree_list *q = _pairs;
   while (true)
     {
-      if (q->next == NULL || q->next->deg > deg)
+      if (q->next == nullptr || q->next->deg > deg)
         {
           // Insert new degree node
           s_pair_degree_list *q1 = new s_pair_degree_list;
           q1->next = q->next;
           q1->deg = deg;
-          q1->pairs = NULL;
+          q1->pairs = nullptr;
           q->next = q1;
           break;
         }
@@ -574,8 +574,8 @@ bool binomial_s_pair_set::next(const int *d,
 // the caller should not free any of the three fields of the
 // s_pair!!
 {
-  if (_pairs->next == NULL) return false;
-  if (d != NULL && _pairs->next->deg > *d) return false;
+  if (_pairs->next == nullptr) return false;
+  if (d != nullptr && _pairs->next->deg > *d) return false;
   s_pair_degree_list *thisdeg = _pairs->next;
   s_pair_lcm_list *thislcm = thisdeg->pairs;
   s_pair_elem *s = thislcm->pairs;
@@ -588,16 +588,16 @@ bool binomial_s_pair_set::next(const int *d,
   result = binomial_s_pair(s->f1, s->f2, thislcm->lcm);
 
   thislcm->pairs = s->next;
-  if (thislcm->pairs == NULL)
+  if (thislcm->pairs == nullptr)
     {
       // Now we must remove this set
       thisdeg->pairs = thislcm->next;
       R->remove_monomial(_prev_lcm);
       _prev_lcm = thislcm->lcm;
-      thislcm->lcm = NULL;
+      thislcm->lcm = nullptr;
       freemem(thislcm);
 
-      if (thisdeg->pairs == NULL)
+      if (thisdeg->pairs == nullptr)
         {
           // Now we must remove this larger degree list
           _pairs->next = thisdeg->next;
@@ -611,15 +611,15 @@ bool binomial_s_pair_set::next(const int *d,
 
 int binomial_s_pair_set::lowest_degree() const
 {
-  if (_pairs->next == NULL) return -1;
+  if (_pairs->next == nullptr) return -1;
   return _pairs->next->deg;
 }
 
 int binomial_s_pair_set::n_elems(int d) const
 {
   s_pair_degree_list *p = _pairs;
-  while (p->next != NULL && p->next->deg < d) p = p->next;
-  if (p->next == NULL || p->next->deg != d) return 0;
+  while (p->next != nullptr && p->next->deg < d) p = p->next;
+  if (p->next == nullptr || p->next->deg != d) return 0;
   return p->next->n_elems;
 }
 
@@ -660,7 +660,7 @@ void binomial_s_pair_set::stats() const
 ///////////////////////
 binomialGB::binomialGB(const binomial_ring *R0, bool bigcell, bool homogprime)
     : R(R0),
-      first(NULL),
+      first(nullptr),
       _max_degree(0),
       // use_bigcell(bigcell),
       is_homogeneous_prime(homogprime)
@@ -672,8 +672,8 @@ binomialGB::~binomialGB()
   // Do nothing much, except maybe clear out stuff
   // so no stray pointers are around
 
-  R = NULL;
-  first = NULL;  // MES: BUG!! We are leaking stuff here!!
+  R = nullptr;
+  first = nullptr;  // MES: BUG!! We are leaking stuff here!!
 }
 
 void binomialGB::enlarge(const binomial_ring *newR) { R = newR; }
@@ -687,7 +687,7 @@ void binomialGB::minimalize_and_insert(binomial_gb_elem *f)
   head.next = first;
   int deg = R->degree(m);
   if (deg > _max_degree) _max_degree = deg;
-  for (p = &head; p->next != NULL; p = p->next)
+  for (p = &head; p->next != nullptr; p = p->next)
     {
       if (R->graded_compare(m, p->next->elem->f.lead) == LT) break;
     }
@@ -696,7 +696,7 @@ void binomialGB::minimalize_and_insert(binomial_gb_elem *f)
   first = head.next;
 
   p = fm;
-  while (p->next != NULL)
+  while (p->next != nullptr)
     {
       if (R->degree(p->next->elem->f.lead) > deg &&
           R->divides(m, p->next->elem->f.lead))
@@ -705,7 +705,7 @@ void binomialGB::minimalize_and_insert(binomial_gb_elem *f)
           gbmin_elem *q = p->next;
           binomial_gb_elem *qe = q->elem;
           p->next = q->next;
-          q->next = NULL;
+          q->next = nullptr;
           qe->smaller = f;
         }
       else
@@ -722,20 +722,20 @@ binomialGB::monomial_list *binomialGB::find_divisor(
 {
   unsigned int mask = ~(R->mask(m));
   int d = R->degree(m);
-  for (monomial_list *p = I; p != NULL; p = p->next)
+  for (monomial_list *p = I; p != nullptr; p = p->next)
     {
-      if (R->degree(p->m) > d) return NULL;
+      if (R->degree(p->m) > d) return nullptr;
       if (mask & p->mask) continue;
       if (R->divides(p->m, m)) return p;
     }
-  return NULL;
+  return nullptr;
 }
 
 binomialGB::monomial_list *binomialGB::ideal_quotient(monomial m) const
 {
   monomial_list *r;
   monomial_list **deglist = newarray(monomial_list *, _max_degree + 1);
-  for (int i = 0; i <= _max_degree; i++) deglist[i] = NULL;
+  for (int i = 0; i <= _max_degree; i++) deglist[i] = nullptr;
 
   for (iterator p = begin(); p != end(); p++)
     {
@@ -747,13 +747,13 @@ binomialGB::monomial_list *binomialGB::ideal_quotient(monomial m) const
       nl->next = deglist[d];
       deglist[d] = nl;
     }
-  monomial_list *result = NULL;
+  monomial_list *result = nullptr;
 
   for (int d = 0; d <= _max_degree; d++)
-    if (deglist[d] != NULL)
+    if (deglist[d] != nullptr)
       {
-        monomial_list *currentresult = NULL;
-        while (deglist[d] != NULL)
+        monomial_list *currentresult = nullptr;
+        while (deglist[d] != nullptr)
           {
             monomial_list *p = deglist[d];
             deglist[d] = p->next;
@@ -777,16 +777,16 @@ binomialGB::monomial_list *binomialGB::ideal_quotient(monomial m) const
                 currentresult = p;
               }
           }
-        if (result == NULL)
+        if (result == nullptr)
           result = currentresult;
-        else if (currentresult != NULL)
+        else if (currentresult != nullptr)
           {
             monomial_list *q;
-            for (q = result; q->next != NULL; q = q->next)
+            for (q = result; q->next != nullptr; q = q->next)
               ;
             q->next = currentresult;
           }
-        currentresult = NULL;
+        currentresult = nullptr;
       }
   freemem(deglist);
   return result;
@@ -805,7 +805,7 @@ void binomialGB::make_new_pairs(binomial_s_pair_set *Pairs,
   monomial m = f->f.lead;
   monomial_list *I = ideal_quotient(m);
 
-  for (monomial_list *q = I; q != NULL; q = q->next)
+  for (monomial_list *q = I; q != nullptr; q = q->next)
     {
       gbmin_elem *ge = q->tag;  // a list of possibles
 
@@ -834,7 +834,7 @@ void binomialGB::make_new_pairs(binomial_s_pair_set *Pairs,
 
 void binomialGB::remove_monomial_list(monomial_list *mm) const
 {
-  while (mm != NULL)
+  while (mm != nullptr)
     {
       R->remove_monomial(mm->m);
       monomial_list *tmp = mm;
@@ -864,13 +864,13 @@ binomial_gb_elem *binomialGB::find_divisor(monomial m) const
 {
   unsigned int mask = ~(R->mask(m));
   int d = R->degree(m);
-  for (gbmin_elem *p = first; p != NULL; p = p->next)
+  for (gbmin_elem *p = first; p != nullptr; p = p->next)
     {
-      if (R->degree(p->elem->f.lead) > d) return NULL;
+      if (R->degree(p->elem->f.lead) > d) return nullptr;
       if (mask & p->mask) continue;
       if (R->divides(p->elem->f.lead, m)) return p->elem;
     }
-  return NULL;
+  return nullptr;
 }
 
 void binomialGB::reduce_monomial(monomial m) const
@@ -885,7 +885,7 @@ bool binomialGB::reduce(binomial &f) const
   while (true)
     {
       binomial_gb_elem *p = find_divisor(f.lead);
-      if (p == NULL)
+      if (p == nullptr)
         {
           reduce_monomial(f.tail);
           return R->normalize(f);
@@ -936,7 +936,7 @@ int binomialGB::n_masks() const
   buffer o;
   unsigned int nmasks = 1;
   masks[0] = first->mask;
-  for (gbmin_elem *p = first; p != NULL; p = p->next)
+  for (gbmin_elem *p = first; p != nullptr; p = p->next)
     {
       o << " " << p->mask;
       bool found = false;
@@ -1000,15 +1000,15 @@ binomialGB_comp *binomialGB_comp::create(const Matrix *m,
   if (collect_syz || n_rows_to_keep > 0)
     {
       ERROR("Groebner basis Algorithm=>Toric cannot keep syzygies");
-      return 0;
+      return nullptr;
     }
   const PolynomialRing *R = m->get_ring()->cast_to_PolynomialRing();
-  if (R == 0)
+  if (R == nullptr)
     {
       ERROR("expected polynomial ring");
-      return 0;
+      return nullptr;
     }
-  binomialGB_comp *result = new binomialGB_comp(R, 0, true, strategy);
+  binomialGB_comp *result = new binomialGB_comp(R, nullptr, true, strategy);
   result->add_generators(m);
   return result;
 }
@@ -1023,8 +1023,8 @@ void binomialGB_comp::remove_gb()
   // remove each element of G
   for (i = 0; i < G.size(); i++) freemem(G[i]);
   // The following is just to ease garbage collection
-  for (i = 0; i < mingens.size(); i++) mingens[i] = NULL;
-  for (i = 0; i < mingens_subring.size(); i++) mingens_subring[i] = NULL;
+  for (i = 0; i < mingens.size(); i++) mingens[i] = nullptr;
+  for (i = 0; i < mingens_subring.size(); i++) mingens_subring[i] = nullptr;
   freemem(R);
 }
 
@@ -1094,7 +1094,7 @@ void binomialGB_comp::process_pair(binomial_s_pair s)
   bool ismin, subringmin;
   binomial f;
 
-  if (s.f2 == NULL)
+  if (s.f2 == nullptr)
     {
       // A generator
       ismin = true;
@@ -1104,7 +1104,7 @@ void binomialGB_comp::process_pair(binomial_s_pair s)
     }
   else
     {
-      if (s.f1->smaller != NULL || s.f2->smaller != NULL)
+      if (s.f1->smaller != nullptr || s.f2->smaller != nullptr)
         {
           if (s.f1->smaller != s.f2 && s.f2->smaller != s.f1)
             {
@@ -1161,7 +1161,7 @@ ComputationStatusCode binomialGB_comp::gb_done() const
 void binomialGB_comp::start_computation()
 {
   binomial_s_pair s;
-  int *deg = 0;
+  int *deg = nullptr;
   if (stop_.always_stop) return;  // don't change status
   if (stop_.stop_after_degree) deg = &stop_.degree_limit->array[0];
   while (Pairs->next(deg, s, top_degree))
@@ -1193,7 +1193,7 @@ Matrix *binomialGB_comp::subring()
   for (int i = 0; i < mingens_subring.size(); i++)
     {
       result.append(R->binomial_to_vector(mingens_subring[i]->f));
-      mingens_subring[i] = NULL;
+      mingens_subring[i] = nullptr;
     }
   mingens_subring.clear();
   return result.to_matrix();
@@ -1211,7 +1211,7 @@ Matrix *binomialGB_comp::subringGB()
 Matrix *binomialGB_comp::reduce(const Matrix *m, Matrix *& /*lift*/)
 {
   ERROR("MES: not implemented yet");
-  return 0;
+  return nullptr;
 }
 
 int binomialGB_comp::contains(const Matrix * /*m*/)
@@ -1244,12 +1244,12 @@ const Matrix *binomialGB_comp::get_gb()
   return result.to_matrix();
 }
 
-const Matrix *binomialGB_comp::get_change() { return 0; }
-const Matrix *binomialGB_comp::get_syzygies() { return 0; }
+const Matrix *binomialGB_comp::get_change() { return nullptr; }
+const Matrix *binomialGB_comp::get_syzygies() { return nullptr; }
 const Matrix /* or null */ *binomialGB_comp::matrix_remainder(const Matrix *m)
 // likely not planned to be implemented
 {
-  return 0;
+  return nullptr;
 }
 M2_bool binomialGB_comp::matrix_lift(
     const Matrix *m,
@@ -1257,8 +1257,8 @@ M2_bool binomialGB_comp::matrix_lift(
     const Matrix /* or null */ **result_quotient)
 // not planned to be implemented
 {
-  *result_remainder = 0;
-  *result_quotient = 0;
+  *result_remainder = nullptr;
+  *result_quotient = nullptr;
   ERROR("rawGBMatrixLift not implemented for toric GB's");
   return false;
 }
