@@ -5,20 +5,22 @@
 #include <vector>
 #include <memory>
 
-#include "aring-gf-flint-big.hpp"
-#include "aring-gf-flint.hpp"
 #include "aring-glue.hpp"
 #include "aring-m2-gf.hpp"
 #include "aring-qq.hpp"
 #include "aring-tower.hpp"
-#include "aring-zz-flint.hpp"
 #include "aring-zzp-ffpack.hpp"
-#include "aring-zzp-flint.hpp"
 #include "aring-zzp.hpp"
 #include "exceptions.hpp"
 #include "polyring.hpp"
 #include "relem.hpp"
 
+// include flint headers last to avoid #1674
+#include "aring-gf-flint-big.hpp"
+#include "aring-gf-flint.hpp"
+#include "aring-zz-flint.hpp"
+#include "aring-zzp-flint.hpp"
+
 const RingQQ *globalQQ;
 
 void initializeRationalRing()
@@ -42,7 +44,7 @@ const Ring /* or null */ *rawARingZZp(unsigned long p)
   if (p <= 1 || p >= 32750)
     {
       ERROR("ZZP: expected a prime number p in range 2 <= p <= 32749");
-      return 0;
+      return nullptr;
     }
   return M2::ConcreteRing<M2::ARingZZp>::create(p);
 }
@@ -58,66 +60,66 @@ static const PolynomialRing * /* or null */ checkGaloisFieldInput(
   // Check that f is monic
   // If any of these fail, then return 0.
   const PolynomialRing *R = f->get_ring()->cast_to_PolynomialRing();
-  if (R == 0)
+  if (R == nullptr)
     {
       ERROR("expected poly ring of the form ZZ/p[x]/(f)");
-      return 0;
+      return nullptr;
     }
   if (R->n_vars() != 1)
     {
       ERROR("expected poly ring of the form ZZ/p[x]/(f)");
-      return 0;
+      return nullptr;
     }
   if (R->n_quotients() != 1)
     {
       ERROR("expected poly ring of the form ZZ/p[x]/(f)");
-      return 0;
+      return nullptr;
     }
   if (R->characteristic() == 0)
     {
       ERROR("expected poly ring of the form ZZ/p[x]/(f)");
-      return 0;
+      return nullptr;
     }
   return R;
 }
 const Ring /* or null */ *rawARingGaloisField1(const RingElement *f)
 {
   const PolynomialRing *R = checkGaloisFieldInput(f);
-  if (R == 0) return 0;  // error message has already been logged
+  if (R == nullptr) return nullptr;  // error message has already been logged
   try
     {
       return M2::ConcreteRing<M2::ARingGFM2>::create(*R, f->get_value());
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 const Ring /* or null */ *rawARingGaloisFieldFlintBig(const RingElement *f)
 {
   const PolynomialRing *R = checkGaloisFieldInput(f);
-  if (R == 0) return 0;  // error message has already been logged
+  if (R == nullptr) return nullptr;  // error message has already been logged
   try
     {
       return M2::ConcreteRing<M2::ARingGFFlintBig>::create(*R, f->get_value());
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
 const Ring /* or null */ *rawARingGaloisFieldFlintZech(const RingElement *f)
 {
   const PolynomialRing *R = checkGaloisFieldInput(f);
-  if (R == 0) return 0;  // error message has already been logged
+  if (R == nullptr) return nullptr;  // error message has already been logged
   try
     {
       return M2::ConcreteRing<M2::ARingGFFlint>::create(*R, f->get_value());
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -138,7 +140,7 @@ const Ring /* or null */ *rawARingGaloisField(int prime, int dimension)
   if (dimension < 0)
     {
       ERROR(" givaroGF/FFPACK: help, dimension is negative ! ");
-      return 0;
+      return nullptr;
     }
   try
     {
@@ -147,7 +149,7 @@ const Ring /* or null */ *rawARingGaloisField(int prime, int dimension)
       if (prime <= 1)
         {
           ERROR("givaroGF/FFPACK: expected a prime number p ");
-          return 0;
+          return nullptr;
         }
 
       //-- NEED_POLYNOMIAL_REPRESENTATION is not usable(namespace problems) and
@@ -168,7 +170,7 @@ const Ring /* or null */ *rawARingGaloisField(int prime, int dimension)
       if (dimension == 1)
         {
           ERROR("maximum modulus = %f\n", M2::ARingZZpFFPACK::getMaxModulus());
-          return 0;
+          return nullptr;
         }
       #if 0
       return M2::ConcreteRing<M2::ARingGFGivaro>::create(prime, dimension);
@@ -182,7 +184,7 @@ const Ring /* or null */ *rawARingGaloisField(int prime, int dimension)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -297,10 +299,10 @@ const Ring /* or null */ *rawARingTower1(const Ring *K, M2_ArrayString names)
     {
       const M2::ConcreteRing<M2::ARingZZpFFPACK> *Kp =
           dynamic_cast<const M2::ConcreteRing<M2::ARingZZpFFPACK> *>(K);
-      if (Kp == 0)
+      if (Kp == nullptr)
         {
           ERROR("expected a base ring ZZ/p");
-          return NULL;
+          return nullptr;
         }
       const M2::ARingZZpFFPACK &A = Kp->ring();
 
@@ -312,7 +314,7 @@ const Ring /* or null */ *rawARingTower1(const Ring *K, M2_ArrayString names)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -323,10 +325,10 @@ const Ring /* or null */ *rawARingTower2(const Ring *R1,
     {
       const M2::ConcreteRing<M2::ARingTower> *K =
           dynamic_cast<const M2::ConcreteRing<M2::ARingTower> *>(R1);
-      if (K == 0)
+      if (K == nullptr)
         {
           ERROR("expected a tower ring");
-          return NULL;
+          return nullptr;
         }
       const M2::ARingTower &A = K->ring();
 
@@ -337,7 +339,7 @@ const Ring /* or null */ *rawARingTower2(const Ring *R1,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -348,10 +350,10 @@ const Ring /* or null */ *rawARingTower3(const Ring *R1,
     {
       const M2::ConcreteRing<M2::ARingTower> *K =
           dynamic_cast<const M2::ConcreteRing<M2::ARingTower> *>(R1);
-      if (K == 0)
+      if (K == nullptr)
         {
           ERROR("expected a tower ring");
-          return NULL;
+          return nullptr;
         }
       const M2::ARingTower &A = K->ring();
 
@@ -364,7 +366,7 @@ const Ring /* or null */ *rawARingTower3(const Ring *R1,
           if (f->get_ring() != R1)
             {
               ERROR("extension element has incorrect base ring");
-              return 0;
+              return nullptr;
             }
           A.from_ring_elem(f1, f->get_value());
           extensions.push_back(f1);
@@ -375,7 +377,7 @@ const Ring /* or null */ *rawARingTower3(const Ring *R1,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return 0;
+      return nullptr;
   }
 }
 
