@@ -13,14 +13,17 @@
 #include "freemod.hpp"
 #include "interface/NAG.h"
 #include "interface/gmp-util.h"
+#include "interface/monoid.h"
 #include "mat.hpp"
 #include "matrix-con.hpp"
 #include "matrix.hpp"
-#include "monoid.hpp"
 #include "mutablemat-defs.hpp"
 #include "relem.hpp"
 #include "ring.hpp"
 #include "ringelem.hpp"
+#include "BasicPolyList.hpp"
+#include "BasicPolyListParser.hpp"
+//#include "matrix-io.hpp"
 
 namespace M2 { class ARingCC; }
 
@@ -28,9 +31,10 @@ const FreeModule *IM2_Matrix_get_target(const Matrix *M) { return M->rows(); }
 const FreeModule *IM2_Matrix_get_source(const Matrix *M) { return M->cols(); }
 int IM2_Matrix_n_rows(const Matrix *M) { return M->n_rows(); }
 int IM2_Matrix_n_cols(const Matrix *M) { return M->n_cols(); }
+
 M2_arrayint IM2_Matrix_get_degree(const Matrix *M)
 {
-  return M->degree_monoid()->to_arrayint(M->degree_shift());
+  return to_degree_vector(M->get_ring()->degree_monoid(), M->degree_shift());
 }
 
 M2_string IM2_Matrix_to_string(const Matrix *M)
@@ -57,14 +61,14 @@ const RingElement /* or null */ *IM2_Matrix_get_entry(const Matrix *M,
       if (r < 0 || r >= M->n_rows())
         {
           ERROR("matrix row index %d out of range 0 .. %d", r, M->n_rows() - 1);
-          return 0;
+          return nullptr;
         }
       if (c < 0 || c >= M->n_cols())
         {
           ERROR("matrix column index %d out of range 0 .. %d",
                 c,
                 M->n_cols() - 1);
-          return 0;
+          return nullptr;
         }
       ring_elem result;
       result = M->elem(r, c);
@@ -72,7 +76,7 @@ const RingElement /* or null */ *IM2_Matrix_get_entry(const Matrix *M,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -189,7 +193,7 @@ const Matrix /* or null */ *IM2_Matrix_remake1(const FreeModule *target,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -207,6 +211,34 @@ const Matrix /* or null */ *IM2_Matrix_random(
   return Matrix::random(R, r, c, fraction_non_zero, special_type);
 }
 
+const Matrix* /* or null */ rawMatrixReadMsolveString(const Ring* R, M2_string contents)
+{
+  try
+    {
+      std::string str = string_M2_to_std(contents);// TODO: this does a full copy.  Perhaps we just have readMsolveIdealContents take a string_view?
+      auto Fs = parseMsolveFromString(str);
+      return toMatrix(R->make_FreeModule(1), Fs);
+    } catch (const exc::engine_error& e)
+    {
+      ERROR(e.what());
+      return nullptr;
+    }
+}
+
+const Matrix* /* or null */ rawMatrixReadMsolveFile(const Ring* R, M2_string filename)
+{
+  try
+    {
+      std::string str = string_M2_to_std(filename);
+      auto Fs = parseMsolveFile(str);
+      return toMatrix(R->make_FreeModule(1), Fs);
+    } catch (const exc::engine_error& e)
+    {
+      ERROR(e.what());
+      return nullptr;
+    }
+}
+
 /////////////////////////////////////////////////////////////////////
 M2_bool IM2_Matrix_is_zero(const Matrix *M) { return M->is_zero(); }
 int  // 1 = true, 0 = false, -1 = error
@@ -235,7 +267,7 @@ const Matrix /* or null */ *IM2_Matrix_concat(const engine_RawMatrixArray Ms)
       if (n == 0)
         {
           ERROR("matrix concat: expects at least one matrix");
-          return 0;
+          return nullptr;
         }
       const FreeModule *F = Ms->array[0]->rows();
       const Ring *R = F->get_ring();
@@ -244,15 +276,15 @@ const Matrix /* or null */ *IM2_Matrix_concat(const engine_RawMatrixArray Ms)
       for (unsigned int i = 0; i < n; i++)
         {
           const Matrix *M = Ms->array[i];
-          if (F->get_ring() != M->get_ring())
+          if (R != M->get_ring())
             {
               ERROR("matrix concat: different base rings");
-              return 0;
+              return nullptr;
             }
           if (F->rank() != M->n_rows())
             {
               ERROR("matrix concat: row sizes are not equal");
-              return 0;
+              return nullptr;
             }
           for (int j = 0; j < M->n_cols(); j++)
             {
@@ -264,7 +296,7 @@ const Matrix /* or null */ *IM2_Matrix_concat(const engine_RawMatrixArray Ms)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -279,7 +311,7 @@ const Matrix /* or null */ *IM2_Matrix_direct_sum(
       if (n == 0)
         {
           ERROR("matrix direct sum: expects at least one matrix");
-          return 0;
+          return nullptr;
         }
       const Matrix *result = Ms->array[0];
       const Ring *R = result->get_ring();
@@ -287,7 +319,7 @@ const Matrix /* or null */ *IM2_Matrix_direct_sum(
         if (R != Ms->array[i]->get_ring())
           {
             ERROR("matrix direct sum: different base rings");
-            return 0;
+            return nullptr;
           }
       for (unsigned int i = 1; i < n; i++)
         result = result->direct_sum(Ms->array[i]);
@@ -296,7 +328,7 @@ const Matrix /* or null */ *IM2_Matrix_direct_sum(
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -308,7 +340,7 @@ const Matrix /* or null */ *IM2_Matrix_tensor(const Matrix *M, const Matrix *N)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -320,7 +352,7 @@ const Matrix /* or null */ *rawModuleTensor(const Matrix *M, const Matrix *N)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -332,7 +364,7 @@ const Matrix /* or null */ *IM2_Matrix_transpose(const Matrix *M)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -346,7 +378,7 @@ const Matrix /* or null */ *IM2_Matrix_reshape(const Matrix *M,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -359,7 +391,7 @@ const Matrix /* or null */ *IM2_Matrix_flip(const FreeModule *F,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -374,7 +406,7 @@ const Matrix /* or null */ *rawWedgeProduct(int p, int q, const FreeModule *F)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -388,7 +420,7 @@ const Matrix /* or null */ *IM2_Matrix_submatrix(const Matrix *M,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -401,7 +433,7 @@ const Matrix /* or null */ *IM2_Matrix_submatrix1(const Matrix *M,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -413,7 +445,7 @@ const Matrix /* or null */ *IM2_Matrix_koszul(int p, const Matrix *M)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -429,13 +461,13 @@ const Matrix /* or null */ *rawKoszulMonomials(int nskew,
       if (M->get_ring() != N->get_ring())
         {
           ERROR("expected same ring");
-          return 0;
+          return nullptr;
         }
       return Matrix::koszul_monomials(nskew, M, N);
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -447,7 +479,7 @@ const Matrix /* or null */ *IM2_Matrix_symm(int p, const Matrix *M)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -539,7 +571,7 @@ const Matrix /* or null */ *IM2_Matrix_homogenize(const Matrix *M,
                                                   int var,
                                                   M2_arrayint wts)
 {
-  return M->homogenize(var, wts);
+  return M->homogenize(var, M2_arrayint_to_stdvector<int>(wts));
 }
 
 const Matrix /* or null */ *rawCoefficients(M2_arrayint vars,
@@ -590,7 +622,7 @@ engine_RawMatrixPairOrNull rawTopCoefficients(const Matrix *M)
   Matrix *coeffs;
   Matrix *monoms;
   coeffs = M->top_coefficients(monoms);
-  if (coeffs == NULL) return NULL;
+  if (coeffs == nullptr) return nullptr;
   engine_RawMatrixPair result = new engine_RawMatrixPair_struct;
   result->a = monoms;
   result->b = coeffs;
@@ -654,7 +686,7 @@ const Matrix /* or null */ *IM2_Matrix_remove_content(const Matrix *M)
 #warning \
     "const Matrix /* or null */ * IM2_Matrix_remove_content(const Matrix *M) -- not implemented yet"
 #endif
-  return NULL;
+  return nullptr;
 }
 
 const Matrix /* or null */ *IM2_Matrix_promote(const FreeModule *newTarget,
@@ -674,14 +706,14 @@ const Matrix /* or null */ *IM2_Matrix_promote(const FreeModule *newTarget,
           else
             {
               ERROR("cannot promote given matrix");
-              return 0;
+              return nullptr;
             }
       mat.compute_column_degrees();
       return mat.to_matrix();
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -703,7 +735,7 @@ const Matrix /* or null */ *IM2_Matrix_lift(int *success_return,
           else
             {
               // ERROR("cannot lift given matrix");
-              return 0;
+              return nullptr;
             }
       mat.compute_column_degrees();
       *success_return = 1;
@@ -711,7 +743,7 @@ const Matrix /* or null */ *IM2_Matrix_lift(int *success_return,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
