@@ -6,12 +6,12 @@
 // TODO: fix this
 #  if defined(__cplusplus)
 class Matrix;
-class Monomial;
+class EngineMonomial;
 class Ring;
 class RingElement;
 #  else
 typedef struct Matrix Matrix;
-typedef struct Monomial Monomial;
+typedef struct EngineMonomial EngineMonomial;
 typedef struct Ring Ring;
 typedef struct RingElement RingElement;
 #  endif
@@ -113,7 +113,7 @@ IM2_RingElement_homogenize(const RingElement *a, int v, M2_arrayint wts);
 
 const RingElement /* or null */ *IM2_RingElement_term(const Ring *R,
                                                       const RingElement *a,
-                                                      const Monomial *m);
+                                                      const EngineMonomial *m);
 /* R must be a polynomial ring, and 'a' an element of the
    coefficient ring of R.  Returns a*m, if this is a valid
    element of R.  Returns NULL if not (with an error message). */
@@ -132,7 +132,7 @@ const RingElement /* or null */ *IM2_RingElement_get_terms(
 const RingElement /* or null */ *IM2_RingElement_get_coeff(
     const Ring *coeffRing, /* ring of the result */
     const RingElement *a,
-    const Monomial *m);
+    const EngineMonomial *m);
 /* Return (as an element of the coefficient ring) the coeff
      of the monomial 'm'.
   */
@@ -141,7 +141,7 @@ const RingElement /* or null */ *IM2_RingElement_lead_coeff(
     const Ring *coeffRing, /* ring of the result */
     const RingElement *a);
 
-const Monomial /* or null */ *IM2_RingElement_lead_monomial(
+const EngineMonomial /* or null */ *IM2_RingElement_lead_monomial(
     int nvars, /* number of variables in an outermost monoid */
     const RingElement *a);
 
@@ -288,7 +288,7 @@ const RingElement /* or null */ *IM2_RingElement_homogenize(
 const RingElement /* or null */ *IM2_RingElement_term(
     const Ring *R,
     const RingElement *a,
-    const Monomial *m); /* drg: connected rawTerm*/
+    const EngineMonomial *m); /* drg: connected rawTerm*/
 /* R must be a polynomial ring, and 'a' an element of the
    coefficient ring of R.  Returns a*m, if this is a valid
    element of R.  Returns NULL if not (with an error message).
@@ -308,7 +308,7 @@ const RingElement /* or null */ *IM2_RingElement_get_terms(
 const RingElement /* or null */ *IM2_RingElement_get_coeff(
     const Ring *coeffRing, /* ring of the result */
     const RingElement *a,
-    const Monomial *m); /* drg: connected rawCoefficient*/
+    const EngineMonomial *m); /* drg: connected rawCoefficient*/
 /* Return (as an element of the coefficient ring) the coeff
    of the monomial 'm'.
 */
@@ -317,7 +317,7 @@ const RingElement /* or null */ *IM2_RingElement_lead_coeff(
     const Ring *coeffRing, /* ring of the result */
     const RingElement *a); /* drg: connected rawLeadCoefficient*/
 
-const Monomial /* or null */ *IM2_RingElement_lead_monomial(
+const EngineMonomial /* or null */ *IM2_RingElement_lead_monomial(
     int nvars,             /* number of variables in an outermost monoid */
     const RingElement *a); /* drg: connected rawLeadMonomial*/
 
