@@ -57,7 +57,12 @@ typedef DMat<M2::ARingCC> DMatCC;
 
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wconversion"
-#include <flint/fmpz_mat.h>
+#include <flint/flint.h>        // for fmpq_numref, fmpz_t
+#include <flint/fmpq_mat.h>     // for fmpq_mat_mul, fmpq_mat_add
+#include <flint/fmpz.h>         // for fmpz_is_pm1, fmpz_clear, fmpz...
+#include <flint/fmpz_mat.h>     // for fmpz_mat_t, fmpz_mat_mul, fmpz_mat_clear
+#include <flint/fq_nmod_mat.h>  // for fq_nmod_mat_mul, fq_zech_mat_mul
+#include <flint/nmod_mat.h>     // for nmod_mat_mul, nmod_mat_add
 #pragma GCC diagnostic pop
 
 #include <iostream>
@@ -859,7 +864,11 @@ inline size_t rowReducedEchelonForm(const DMatGFFlintBig& A,
                                     DMatGFFlintBig& result_rref)
 {
   DMatGFFlintBig A1(A);
+#if __FLINT_RELEASE >= 30100
+  long rank = fq_nmod_mat_rref(A1.fq_nmod_mat(), A1.fq_nmod_mat(), A.ring().flintContext());
+#else
   long rank = fq_nmod_mat_rref(A1.fq_nmod_mat(), A.ring().flintContext());
+#endif
   result_rref.swap(A1);
   return rank;
 }
@@ -920,7 +929,11 @@ inline size_t rowReducedEchelonForm(const DMatGFFlint& A,
                                     DMatGFFlint& result_rref)
 {
   DMatGFFlint A1(A);
+#if __FLINT_RELEASE >= 30100
+  long rank = fq_zech_mat_rref(A1.fq_zech_mat(), A1.fq_zech_mat(), A.ring().flintContext());
+#else
   long rank = fq_zech_mat_rref(A1.fq_zech_mat(), A.ring().flintContext());
+#endif
   result_rref.swap(A1);
   return rank;
 }
@@ -989,7 +1002,7 @@ inline size_t rank(const DMatQQFlint& A)
   // that matrix.
   fmpz_mat_t m1;
   fmpz_mat_init(m1, A.numRows(), A.numColumns());
-  fmpq_mat_get_fmpz_mat_rowwise(m1, NULL, A.fmpq_mat());
+  fmpq_mat_get_fmpz_mat_rowwise(m1, nullptr, A.fmpq_mat());
   // fmpz_mat_print_pretty(m1);
   size_t rk = fmpz_mat_rank(m1);
   fmpz_mat_clear(m1);
@@ -1019,7 +1032,7 @@ inline size_t nullSpace(const DMatQQFlint& A, DMatQQFlint& result_nullspace)
   fmpz_mat_t m2;
   fmpz_mat_init(m1, A.numRows(), A.numColumns());
   fmpz_mat_init(m2, A.numColumns(), A.numColumns());
-  fmpq_mat_get_fmpz_mat_rowwise(m1, NULL, A.fmpq_mat());
+  fmpq_mat_get_fmpz_mat_rowwise(m1, nullptr, A.fmpq_mat());
   // fmpz_mat_print_pretty(m1);
   size_t nullity = fmpz_mat_nullspace(m2, m1);
   // now copy the first 'nullity' columns into result_nullspace
