@@ -1,5 +1,5 @@
 ;;; M2.el --- Major mode for editing Macaulay2 source core -*- lexical-binding: t -*-
-;; Version: 1.24.05
+;; Version: 1.24.11
 ;; Keywords: languages Macaulay2
 ;; URL: https://github.com/Macaulay2/M2-emacs
 ;; Package-Requires: ((emacs "24.1"))
@@ -50,6 +50,7 @@
   "Major mode for interacting with a Macaulay2 process.\n\n\\{M2-comint-mode-map}"
   (M2-common)
   (setq comint-prompt-regexp M2-comint-prompt-regexp)
+  (add-hook 'comint-input-filter-functions #'M2-comint-forget-errors nil t)
   (add-hook 'comint-preoutput-filter-functions 'M2-info-help nil t)
   (setq-local compilation-error-regexp-alist M2-error-regexp-alist)
   (setq-local compilation-transform-file-match-alist
@@ -353,8 +354,8 @@ can be executed with \\[M2-send-to-program]."
     ;; error messages, e.g.,
     ;; i1 : load "packages/Macaulay2Doc/demo1.m2"; g 2
     ;; packages/Macaulay2Doc/demo1.m2:8:12:(3):[2]: error: division by zero
-    ;;  (1                                           1)   (2      2)   (3      3)
-    ("\\(?:\\(?1:[^\\[:\n\r\s]+\\)\\|\"\\(?1:.+\\)\"\\):\\([0-9]+\\):\\([0-9]+\\):([0-9]+):\\[[0-9]+\\]"
+    ;;  (1                                                           1)   (2      2)   (3      3)
+    ("\\(?:\\(?1:[[:alnum:]/._][[:alnum:]/._-]*\\)\\|\"\\(?1:.+\\)\"\\):\\([0-9]+\\):\\([0-9]+\\):([0-9]+):\\[[0-9]+\\]"
      1 2 3)
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     ;; net(FilePosition) (debugging.m2) ;;
@@ -362,21 +363,20 @@ can be executed with \\[M2-send-to-program]."
     ;; start & end line/column numbers, e.g.,:
     ;; i1 : locate (rank, Matrix)
     ;; o1 = m2/matrix1.m2:663:19-666:20
-    ;;  (1                                           1)   (2      2)   (3      3)   (4      4)   (5      5)
-    ("\\(?:\\(?1:[^\\[:\n\r\s]+\\)\\|\"\\(?1:.+\\)\"\\):\\([0-9]+\\):\\([0-9]+\\)-\\([0-9]+\\):\\([0-9]+\\)"
+    ;;  (1                                                           1)   (2      2)   (3      3)   (4      4)   (5      5)
+    ("\\(?:\\(?1:[[:alnum:]/._][[:alnum:]/._-]*\\)\\|\"\\(?1:.+\\)\"\\):\\([0-9]+\\):\\([0-9]+\\)-\\([0-9]+\\):\\([0-9]+\\)"
      1 (2 . 4) (3 . 5) 0)
     ;; no end line/column numbers, e.g.,:
     ;; i2 : locate makeDocumentTag rank
     ;; o2 = ../Macaulay2Doc/functions/rank-doc.m2:34:0
-    ;;  (1                                           1)   (2      2)   (3      3)
-    ("\\(?:\\(?1:[^\\[:\n\r\s]+\\)\\|\"\\(?1:.+\\)\"\\):\\([0-9]+\\):\\([0-9]+\\)"
+    ;;  (1                                                          1)   (2      2)   (3      3)
+    ("\\(?:\\(?1:[[:alnum:]/._][[:alnum:]/._-]*\\)\\|\"\\(?1:.+\\)\"\\):\\([0-9]+\\):\\([0-9]+\\)"
      1 2 3 0))
   "Regular expressions for matching file positions in Macaulay2 output.")
 
 (defvar M2-transform-file-match-alist
   '(("^stdio$" nil)
     ("^currentString$" nil)
-    ("^Macaulay2/Core/startup\\.m2\\.in$" nil)
     ("^[0-9][0-9]$" nil))
   "List of filenames not to match in Macaulay2 output.")
 
@@ -423,7 +423,9 @@ SEND-TO-BUFFER."
   (M2--send-to-program-helper send-to-buffer
 			      (save-excursion (M2-to-end-of-prompt) (point))
 			      (line-end-position))
-  (forward-line))
+  (forward-line)
+  ;; add a newline after a nonempty line at the end of the buffer
+  (when (and (eobp) (not (bolp))) (newline)))
 
 (defun M2-send-to-program (send-to-buffer)
      "Send the current line except for a possible prompt, or the region, if the
@@ -525,6 +527,15 @@ Otherwise, send the input to Macaulay2."
 	(substring string end))
     string))
 
+(defun M2-comint-forget-errors (string)
+  "Run `compilation-forget-errors' to flush compilation mode's cache.
+Otherwise, jumping to source will go to the wrong location when a file has
+been modified and reloaded.  STRING is ignored, but we need it so that this
+function can be added to `comint-input-filter-functions' so that it is run each
+time we send new input to the M2 process."
+  (ignore string)
+  (compilation-forget-errors))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; M2-mode
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
